%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% This file provide some example of aggregates under ATCLP %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% minimum
% Compute the minimum.
entails(min,A,B) :- A >= B.
join(min,A,B,B) :- A >= B.
join(min,A,B,A) :- A < B.


%% maximum
% Compute the maximum.
entails(max,A,B) :- A =< B.
join(max,A,B,B) :- A =< B.
join(max,A,B,A) :- A > B.


%% minimum of a pair: Entail-based aggregate version.  \code{join}
%% fails if a pair does not entail or is not entailed by the
%% other. I.e., \code{(1,2)} $\sqcup_{min}$ \code{(2,1)} is not
%% defined.

% (min,min0 pareto frontier.
entails(min,(A1,A2),(B1,B2)) :- 
    entails(min,A1,B1),
    entails(min,A2,B2).
join(min,ParA,ParB,ParB) :- 
    entails(min,ParA,ParB).
join(min,ParA,ParB,ParA) :- 
    entails(min,ParB,ParA).


%% minimum of a pair: Join-based aggregate version, The \code{join}
%%  returns the least upper bound pair that is entailed by them. I.e.,
%%  \code{(1,1)} $=$ \code{(1,2)} $\sqcup_{min}$ \code{(2,1)}.

% Compute the minimum pair.
entails(min,(A1,A2),(B1,B2)) :- 
    entails(min,A1,B1),
    entails(min,A2,B2).
join(min,,(A1,A2),(B1,B2),(C1,C2)) :- 
    join(min,A1,B1,C1),
    join(min,A2,B2,C2).


%% variant/index: aggregate by default \code{'_'}.\\[-2em]

% Equal up to renaming.
:- use_module(terms_check).
entails(index,A,B) :- variant(A,B).
join(index,A,B,B) :- variant(A,B).


%% subsumption
% Subsumed by (an instance).
:- use_module(terms_check).
entails(sub,A,B) :- instance(A,B).
join(sub,A,B,B) :- instance(A,B).
join(sub,A,B,A) :- instance(B,A).


%% set entailment
% Set inclusion (a sublists).
:- use_module(lists).
entails(set,A,B) :- sublist(A,B).
join(set,A,B,B) :- sublist(A,B).
join(set,A,B,A) :- sublist(B,A).


%% set union
% Compute the union of sets.
:- use_module(lists).
entails(set,A,B) :- sublist(A,B).
join(set,A,B,C) :- union(A,B,C).


%% first
% Keep the first (nt).
entails(first,A,B) :- true.
join(first,A,B,B) :- true.


%% last
% Keep the last.
:- use_module(terms_check).
entails(last,A,B) :- variant(A,B).
join(last,A,B,A) :- true.


%% all
% Keep all the answers.
entails(all,A,B) :- false.


%% addition
% Compute the sum.
entails(sum,A,B) :- false.
join(sum,A,B,C) :- C is A + B.


%% multiplication
% Compute the product.
entails(prod,A,B) :- false.
join(prod,A,B,C) :- C is A * B.


%% threshold(Theta)
% Theta is passed as parameter,% specifies the precision limit.
entails(threshold(Theta),A,B) :- 
     A < Theta * B.


%% Intervals
% Join overlapped intervals.
entails(interval,(A1,A2),)(B1,B2)) :- 
       A1 >= B1, A2 =< B2.
join(interval,(A1,A2),(B1,B2),(C1,C2)) :-
       A1 =< B1, B1 =< A2, 
       min(A1,B1,C1), max(A2,B2,C2), !.
join(interval,(A1,A2),(B1,B2),(C1,C2)) :-
       B1 =< A1, A1 =< B2, 
       min(A1,B1,C1), max(A2,B2,C2).

