:- module(game_atclp_02_translation, _).

:- include(aggregate_rt).


%% ORIGINAL PROGRAM %%
% :- table total_fun(max), reach(_, _, max, max).
% total_fun(F) :-
% 	reach(initial, end, _, F).
% reach(GameA, GameB, Tf, Ff) :-
% 	reach(GameA, GameZ, T1, F1),
% 	edge(GameZ, GameB, T2, F2),
% 	Ff is F1 + F2,
% 	Tm is T1 + T2, Tm >= 0,
% 	cap(Cap), (Tm > Cap -> Tf is Cap ; Tf is Tm).
% reach(GameA, GameB, T, F) :-
% 	edge(GameA, GameB, T, F).


%% ATLCP TRANSFORMATION %%
:- table '$aggr_total_fun'/1, '$aggr_reach'/4.

total_fun(Att1) :-
	put(V1, (max,Free1)),
	'$aggr_total_fun'(V1),
	( ground(Att1) -> entails(first,Att1,Free1)
	; Att1 = Free1 ).

'$aggr_total_fun'(V1) :-
	get(V1,(max,Att1)),
	Att1 = F,
	reach(initial, end, _, F).

reach(GameA, GameB, Att1, Att2) :-
	put(V1, (max,Free1)),
	put(V2, (max,Free2)),
	'$aggr_reach'(GameA, GameB, V1, V2),
	( ground(Att1) -> entails(first,Att1,Free1)
	; Att1 = Free1 ),
	( ground(Att2) -> entails(first,Att2,Free2)
	; Att2 = Free2 ).

'$aggr_reach'(GameA, GameB, V1, V2) :-
	get(V1, (max,Att1)),
	get(V2, (max,Att2)),
	Att1 = Tf,
	Att2 = Ff,
	reach(GameA, GameZ, T1, F1),
	edge(GameZ, GameB, T2, F2),
	Ff is F1 + F2,
	Tm is T1 + T2, Tm >= 0,
	cap(Cap), (Tm > Cap -> Tf is Cap ; Tf is Tm).
'$aggr_reach'(GameA, GameB, V1, V2) :-
	get(V1, (max,Att1)),
	get(V2, (max,Att2)),
	Att1 = T,
	Att2 = F,
	edge(GameA, GameB, T, F).
%% ATLCP TRANSFORMATION %%


%% Aggregate definition %%
entails(max,A,B) :-
	A =< B.

edge(initial,      g(1, no),     T,  0) :- cap(T).
edge(g(Game, yes), end,          0,  0) :- num_g(Game).
edge(g(Game, _),   g(Game, yes), -1, F) :- fun(Game, F).
edge(g(Game, yes), g(Game1, no), T,  0) :-
	Game1 is Game + 1, num_g(Games), Game1 =< Games,
	refill(T).


:- include(data_02).

main(_) :-
	statistics(runtime, _),
	total_fun(X),
	statistics(runtime,[_,T]),
	nl, display(total_fun(X)),
	display('       Time = '),display(T),display(' ms'),nl,nl,nl.

