

:- include(aggregate_rt).


%% ORIGINAL PROGRAM %%
% :- table minimax(_, first, best).
% minimax(Pos, NextPos, (Pos,Val)) :-
% 	move(Pos, NextPos),
% 	minimax(NextPos, _, (NextPos, Val)).
% minimax(Pos, Pos, (Pos,Val)) :-
% 	\+ move(Pos, _),
% 	utility(Pos,Val).
%% ORIGINAL PROGRAM %%


%% ATLCP TRANSFORMATION %%
:- table '$minimax'/3.

minimax(Pos, Att1, Att2) :-
	put(V1,(first,Free1)),
	put(V2,(best,Free2)),
	'$minimax'(Pos, V1, V2),
	( ground(Att1) -> entails(first,Att1,Free1)
	; Att1 = Free1 ),
	( ground(Att2) -> entails(first,Att2,Free2)
	; Att2 = Free2 ).

'$minimax'(Pos,  V1, V2) :-
	get(V1,(first,Att1)),
	get(V2,(best,Att2)),
	Att1 = NextPos,
	Att2 = (Pos,Val),
	move(Pos, NextPos),
	minimax(NextPos, _, (NextPos,Val)).

'$minimax'(Pos, V1, V2) :-
	get(V1,(first,Att1)),
	get(V2,(best,Att2)),
	Att1 = Pos,
	Att2 = (Pos,Val),
	\+ move(Pos, _),
	utility(Pos, Val).
%% ATLCP TRANSFORMATION %%


%% Aggregate definition %%
entails(best,(Pos,ValA),(Pos,ValB)) :-      % New Pos better than Previous
	min_to_move(Pos),
	ValB =< ValA.                        % MAX prefers the greater value
	
entails(best,(Pos,ValA),(Pos,ValB)) :-      % New Pos better than Previous
	max_to_move(Pos),
	ValB >= ValA.                        % MIN prefers the lesser value

entails(first,_,_) :- true.
