

:- use_package(tabling).
:- table best/3.
	

% Pos has successors
minimax(Pos, BestNextPos, Val) :-                     
	bagof(NextPos, move(Pos, NextPos), NextPosList),
	best(NextPosList, BestNextPos, Val), !.

% Pos has no successors
minimax(Pos, _, Val) :-
	utility(Pos, Val).



% There is no more position to compare
best([Pos], Pos, Val) :-                                
	minimax(Pos, _, Val), !.

% There are other positions to compare
best([Pos1 | PosList], BestPos, BestVal) :-             
	minimax(Pos1, _, Val1),
	best(PosList, Pos2, Val2),
	betterOf(Pos1, Val1, Pos2, Val2, BestPos, BestVal).



betterOf(Pos0, Val0, _, Val1, Pos0, Val0) :-   % Pos0 better than Pos1
	min_to_move(Pos0),                         % MIN to move in Pos0
	Val0 > Val1, !.                            % MAX prefers the greater value

betterOf(Pos0, Val0, _, Val1, Pos0, Val0) :-   % Pos0 better than Pos1
	max_to_move(Pos0),                         % MAX to move in Pos0
	Val0 < Val1, !.                            % MIN prefers the lesser value

betterOf(_, _, Pos1, Val1, Pos1, Val1).        % Otherwise Pos1 better than Pos0



