:- module(player4x4b_atclp, _).


:- include(minimax_atclp).
:- include(tictactoe4x4).

% bestMove(+Pos, -NextPos)
% Compute the best Next Position from Position Pos
% with minimax or alpha-beta algorithm.
bestMove(Pos, NextPos) :-
	statistics(runtime,_),
	minimax(Pos, NextPos, _),
	statistics(runtime,[_,T]),
	tabling_stats,     %% Print the memory usage of tabling
	print('time = '),print(T),nl,nl,nl.


% play
% Start the game.
play_ :-
    nl,
    write('===================='), nl,
	  write('= ATCLP  TicTacToe ='), nl,
	  write('===================='), nl, nl,
	  write('Rem : x starts the game'), nl,
	  playAskColor.
	
	
	
% playAskColor
% Ask the color for the human player and start the game with it.
playAskColor :-
	  nl, write('Color for human player ? (x or o)'), nl,
	  %	  read(Player), nl,
	  Player = o,
	  (
	    Player \= o, Player \= x, !,     % If not x or o -> not a valid color
	    write('Error : not a valid color !'), nl,
	    playAskColor                     % Ask again
	    ;
	      EmptyBoard = [
 			        0, 0, 0, 0,
			        o, 0, 0, x,
			        0, 0, 0, 0,
			        0, 0, 0, 0  ],
	    show(EmptyBoard), nl,
	
	    % Start the game with color and emptyBoard
	    play([x, play, EmptyBoard], Player)
	  ).


% play(+Position, +HumanPlayer)
% If next player to play in position is equal to HumanPlayer -> Human must play
% Ask to human what to do.
play([Player, play, Board], Player) :- !,
    nl, write('Next move ?'), nl,
    %    read(Pos), nl,                                  % Ask human where to play
    Pos = exit,
    (
	Pos == exit ->
	halt
    ;
      humanMove([Player, play, Board], [NextPlayer, State, NextBoard], Pos), !,
      show(NextBoard),
      (
        State = win, !,                             % If Player win -> stop
        nl, write('End of game : '),
        write(Player), write(' win !'), nl, nl
        ;
        State = draw, !,                            % If draw -> stop
        nl, write('End of game : '),
        write(' draw !'), nl, nl
        ;
        play([NextPlayer, play, NextBoard], Player) % Else -> continue the game
      )
      ;
      write('-> Bad Move !'), nl,                % If humanMove fail -> bad move
      play([Player, play, Board], Player)        % Ask again
    ).



% play(+Position, +HumanPlayer)
% If it is not human who must play -> Computer must play
% Compute the best move for computer with minimax or alpha-beta.
play([Player, play, Board], HumanPlayer) :-
    nl, write('Computer play : '), nl, nl,
    % Compute the best move
    bestMove([Player, play, Board], [NextPlayer, State, BestSuccBoard]),
    show(BestSuccBoard),
    (
      State = win, !,                                 % If Player win -> stop
      nl, write('End of game : '),
      write(Player), write(' win !'), nl, nl
      ;
      State = draw, !,                                % If draw -> stop
      nl, write('End of game : '), write(' draw !'), nl, nl
      ;
      % Else -> continue the game
      play([NextPlayer, play, BestSuccBoard], HumanPlayer)
    ).



% nextPlayer(X1, X2)
% True if X2 is the next player to play after X1.
nextPlayer(o, x).
nextPlayer(x, o).

% When human play
humanMove([X1, play, Board], [X2, State, NextBoard], Pos) :-
    nextPlayer(X1, X2),
    set1(Pos, X1, Board, NextBoard),
    (
      winPos(X1, NextBoard), !, State = win ;
      drawPos(X1,NextBoard), !, State = draw ;
      State = play
    ).



% set1(+Elem, +Pos, +List, -ResList).
% Set Elem at Position Pos in List => Result in ResList.
% Rem : counting starts at 1.
set1(1, E, [X|Ls], [E|Ls]) :- !, X = 0.

set1(P, E, [X|Ls], [X|L2s]) :-
    number(P),
    P1 is P - 1,
    set1(P1, E, Ls, L2s).


% show(+Board)
% Show the board to current output.
show([
	X01, X02, X03, X04,
	X11, X12, X13, X14,
	X21, X22, X23, X24,
	X31, X32, X33, X34
	]) :-
    write('   '), show2(X01),
    write(' | '), show2(X02),
    write(' | '), show2(X03),
    write(' | '), show2(X04), nl,
    write('  ---------------'), nl,
    write('   '), show2(X11),
    write(' | '), show2(X12),
    write(' | '), show2(X13),
    write(' | '), show2(X14), nl,
    write('  ---------------'), nl,
    write('   '), show2(X21),
    write(' | '), show2(X22),
    write(' | '), show2(X23),
    write(' | '), show2(X24), nl,
    write('  ---------------'), nl,
    write('   '), show2(X31),
    write(' | '), show2(X32),
    write(' | '), show2(X33),
    write(' | '), show2(X34), nl.



% show2(+Term)
% Write the term to current outupt
% Replace 0 by ' '.
show2(X) :-
    X = 0, !,
    write(' ').

show2(X) :-
	write(X).


main(_) :- play_.



size(3).

create_emptyBoard(Board) :-
	size(N),
	Size is N * N,
	create_emptyBoard_(Size,Board).
create_emptyBoard_(0,[]).
create_emptyBoard_(N1,[0|B]) :-
	N1 > 0,
	N is N1 -  1,
	create_emptyBoard_(N,B).
	