%:- module(tictactoe, [move/2,min_to_move/1,max_to_move/1,utility/2,winPos/2,drawPos/2]).

% move(+Pos, -NextPos)
% True if there is a legal (according to rules) move from Pos to NextPos.
move([X1, play, Board], [X2, win, NextBoard]) :-
    nextPlayer(X1, X2),
    move_aux(X1, Board, NextBoard),
    winPos(X1, NextBoard), !.

move([X1, play, Board], [X2, draw, NextBoard]) :-
    nextPlayer(X1, X2),
    move_aux(X1, Board, NextBoard),
    drawPos(X1,NextBoard), !.

move([X1, play, Board], [X2, play, NextBoard]) :-
    nextPlayer(X1, X2),
    move_aux(X1, Board, NextBoard).

% move_aux(+Player, +Board, -NextBoard)
% True if NextBoard is Board whith anx empty case replaced by Player mark.
move_aux(P, [0|Bs], [P|Bs]).

move_aux(P, [B|Bs], [B|B2s]) :-
    move_aux(P, Bs, B2s).


% min_to_move(+Pos)
% True if the next player to play is the MIN player.
min_to_move([o, _, _]).

% max_to_move(+Pos)
% True if the next player to play is the MAX player.
max_to_move([x, _, _]).

% utility(+Pos, -Val) :-
% True if Val the the result of the evaluation function at Pos.
% We will only evaluate for final position.
% So we will only have MAX win, MIN win or draw.
% We will use  1 when MAX win
%             -1 when MIN win
%              0 otherwise.
utility([o, win, _], 1).       % Previous player (MAX) has win.
utility([x, win, _], -1).      % Previous player (MIN) has win.
utility([_, draw, _], 0).


% drawPos(+Player, +Board)
% True if the game is a draw.
drawPos(_,Board) :-
    \+ member(0, Board).


% winPos(+Player, +Board)
% True if Player win in Board.
winPos(P, [
	P, P, P, P,
	_, _, _, _,
	_, _, _, _,
	_, _, _, _
	]).
winPos(P, [
	_, _, _, _,
	P, P, P, P,
	_, _, _, _,
	_, _, _, _
	]).
winPos(P, [
	_, _, _, _,
	_, _, _, _,
	P, P, P, P,
	_, _, _, _
	]).
winPos(P, [
	_, _, _, _,
	_, _, _, _,
	_, _, _, _,
	P, P, P, P
	]).
winPos(P, [
	P, _, _, _,
	P, _, _, _,
	P, _, _, _,
	P, _, _, _
	]).
winPos(P, [
	_, P, _, _,
	_, P, _, _,
	_, P, _, _,
	_, P, _, _
	]).
winPos(P, [
	_, _, P, _,
	_, _, P, _,
	_, _, P, _,
	_, _, P, _
	]).
winPos(P, [
	_, _, _, P,
	_, _, _, P,
	_, _, _, P,
	_, _, _, P
	]).
winPos(P, [
	P, _, _, _,
	_, P, _, _,
	_, _, P, _,
	_, _, _, P
	]).
winPos(P, [
	_, _, _, P,
	_, _, P, _,
	_, P, _, _,
	P, _, _, _
	]).